/*
	BromScript - On Runtime Scripting Language.
    Copyright (C) 2012 - 2014  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "BSLibGlobal.h"
#include "../SIF.h"
#include "../Objects/ArgumentData.h"
#include "../Objects/Variable.h"

#ifdef _MSC_VER
#include <Windows.h>
#else
#include <unistd.h>
#endif

using namespace Scratch;

namespace BromScript{
	namespace Libaries{
		namespace Global{
			BS_FUNCTION(Environment){
#ifdef _MSC_VER
				return Converter::ToVariable("Windows");
#else
				return Converter::ToVariable("Linux");
#endif
			}

			BS_FUNCTION(Sleep){
				if (!args->CheckType(0, VariableType::Number, true)) return null;
#ifdef _MSC_VER
		::Sleep((int)args->GetNumber(0));
#else
		sleep(1);
#endif
				return null;
			}

			BS_FUNCTION(Call){
				if (!args->CheckType(0, VariableType::Function, true)) return null;

				Function* tryfunc = args->GetFunction(0);
				List<Variable*> vars;

				for(int i = 1; i < args->Count; i++)
					vars.Add(args->GetVariable(i));

				tryfunc->Run(&vars);

				return Converter::ToVariable(!args->ErrorOccured());
			}

			BS_FUNCTION(Print){
				CString out;
				for (int i = 0; i < args->Count; i++){
					if (i > 0)
						out += '\t';

					out += Converter::VariableToString(args->GetVariable(i));
				}

				out += "\n";
				printf("%s", out.str_szBuffer);
				return null;
			}

			BS_FUNCTION(ToString){
				return Converter::ToVariable(Converter::VariableToString(args->GetVariable(0)));
			}

			BS_FUNCTION(TypeOf){
				return Converter::ToVariable(Converter::TypeToString(bsi, args->GetVariable(0)));
			}

			BS_FUNCTION(Include){
				if (!args->CheckType(0, VariableType::String, true)) return null;

				bsi->DoFile(args->GetString(0));
				return null;
			}

#ifdef _MSC_VER
			BS_FUNCTION(IncludeDll){
				if (!args->CheckType(0, VariableType::String, true)) return null;

				bsi->DoDLL(args->GetString(0));
				return null;
			}
#else
			BS_FUNCTION(IncludeSO){
				if (!args->CheckType(0, VariableType::String, true)) return null;

				bsi->DoSO(args->GetString(0));
				return null;
			}
#endif

			BS_FUNCTION(SetReadOnly){
				if (args->Count != 1){
					args->Error("Expected variable, got nothing!");
					return null;
				}
				
				args->GetVariable(0)->ReadOnly = true;
				return null;
			}

			BS_FUNCTION(Error){
				if (!args->CheckType(0, VariableType::String, true)) return null;

				args->Error(args->GetString(0));
				return null;
			}
		}
	}
}