/*
	BromScript - On Runtime Scripting Language.
    Copyright (C) 2012 - 2014  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef BROMSCRIPT_COMPILER_INCLUDED
#define BROMSCRIPT_COMPILER_INCLUDED

#include "../SIF.h"
#include "../Misc.h"
#include "../Objects/List.h"
#include "../Objects/ByteWriter.h"
#include "../Scratch/CString.h"

namespace BromScript {
	class Compiler
	{
	public:
		static byte* Run(const char* filename, const char* chunk, int chunklen, int* outbytecodelength, bool writelinenumbers = true, bool writeheader = true);

	private:
		Scratch::CString Filename;
		int CurrentStatmentLine;
		int CurrentLineParent;
		int CurrentPos;
		bool WriteLineNumbers;

		ByteWriter Writer;

		Scratch::CDictionary<Scratch::CString, Scratch::CString> Defines;
		Scratch::CDictionary<Scratch::CString, Scratch::CString> Sets;

		int ChunkSize;
		char* CurrentChunk;

		Scratch::CString* Peekline();
		Scratch::CString* ReadLine();
		Scratch::CString Unescape(Scratch::CString str);

		int SkipIfBlock();
		int DoProcess();
		void DoDefines();
		bool EndOfLine(int i);
		bool PastEndOfLine(int i);
		int GetCurrentLine(char* haystack,  int cpos);

		byte* GetArgumentData(Scratch::CString line, int* size);
		void GetFunctionData(Scratch::CString line, char splitter, List<Scratch::CString>* ret);
		void GetIndexes(Scratch::CString line, List<Scratch::CString>* ret);
		Scratch::CString FindString(Scratch::CString line, Scratch::CString start, Scratch::CString end, int startpos);
		Scratch::CString* FindStringSpecial(Scratch::CString line, Scratch::CString start, Scratch::CString end, int startpos);
		int FindChar(Scratch::CString line, int startpos, char* tofind, int tofindlen);
		
		void ThrowError(Scratch::CString reason);

		void WriteFunction(Scratch::CString line);
		void WriteClass(Scratch::CString line);
		void WriteIf(Scratch::CString line);
		void WriteFor(Scratch::CString line);
		void WriteForeach(Scratch::CString line);
		void WriteWhile(Scratch::CString line);
		void WriteSetVar(Scratch::CString line);
		void WriteReturn(Scratch::CString line);
		void WriteMisc(Scratch::CString line);

		Compiler(Scratch::CString filename, const char* chunk, int chunklen, bool addcurline);
		~Compiler(void);
		
		byte* Run(int* outbytecodelength, List<Scratch::CString*>* strtbl);
		byte* Run(int* outbytecodelength, int currentlineparent, List<Scratch::CString*>* strtbl);
		void AddDefine(Scratch::CString key, Scratch::CString value);
		void AddSet(Scratch::CString key, Scratch::CString value);
	};
}

#endif