﻿/*
BromScript - On Runtime Scripting Language.
Copyright (C) 2012 - 2013  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "BromScript.h"

using namespace BromScript;
using namespace Scratch;

BS_ERRORFUNCTION(errfunc){
	ArgumentData args;
	args.AddVariable(bsi->GetVar("Console")->GetTable()->Get("ColorRed"));
	bsi->GetVar("Console")->GetTable()->Get("SetForeColor")->GetFunction()->Run(&args, bsi->GetVar("Console"));

	printf("BromScript error!\n");

	for (int i = 0; i < stacksize; i++){
		printf("SID: %i, File: %s, Name: %s, Line: %d\n", i, stack[i].Filename.str_szBuffer, stack[i].Name.str_szBuffer, stack[i].LineNumber);
	}

	printf("Message: %s\n", msg);

	bsi->GetVar("Console")->GetTable()->Get("SetDefaultColors")->GetFunction()->Run(&args, bsi->GetVar("Console"));
	
	fflush(stdout);
}

void printHelp(){
	printf("-c -- compile flag, setting this disables -r, for compile AND run, put -r after -c, not enabled by default\n");
	printf("-r -- execute flag, enabled by default\n");
	printf("-l -- compile with linenumbers enabled (required for debugging), not enabled by default\n");
	printf("-h -- write bromscript header in output, not enabled by default\n");
	printf("-d -- force debug mode pre-execute (this forces -l), not enabled by default\n");
	printf("-o -- change output destination (use this infront of targeted files), defaults to out.cbs\n");
	printf("-s -- sleep after code end until newline encountered on stdin. not enabled by default\n");
	printf("\n");
	printf("Example:\n");
	printf("bsexec.exe -c -l -h -d -s -o  \"out1.bs\" \"C:/in1.bs\" -o \"out2.bs\" \"C:/in2.bs\" -o \"out3.bs\" \"C:/in3.bs\"\n");

	char tmp[256];
	fgets (tmp, 100, stdin);
}

int main(int argc, char* argv[]) {
	if (argc == 1){
		printHelp();
		return 0;
	}

	setbuf(stdout, NULL);

	bool shouldrun = true;
	bool shouldcompile = false;
	bool writelines = false;
	bool writeheader = false;
	bool shoulddebug = false;
	bool sleepatend = false;
	CString outname = "out.cbs";

	for (int i = 1; i < argc; i++){
		CString str(argv[i]);

		if (str.StartsWith("-")){
			switch(str.str_szBuffer[1]){
			case 'c': shouldcompile = true; shouldrun = false; break;
			case 'r': shouldrun = true; break;
			case 'l': writelines = true; break;
			case 's': sleepatend = true; break;
			case 'h': writeheader = true; break;
			case 'd': shoulddebug = true; writelines = true; break;
			case 'o': outname = argv[++i]; break;
			case '?':
				printHelp();
				break;
			}
		}else{
			CString nicename = str.Replace("\\", "/");
			if (nicename.Contains("/"))
				nicename = nicename.Substring(nicename.LastIndexOf('/') + 1);

			if (shouldcompile){
				printf("NOTICE:Compiling File:%s\n", nicename.str_szBuffer);

				CFileStream cfs;
				if (!cfs.Open(str, "rb")){
					printf("ERROR: FileNotFound\n");
					return 1;
				}

				int fsize = (int)cfs.Size();

				char* buff = (char*)cfs.Read(fsize);

				void* bytecode = null;

				try{
					bytecode = Compiler::Run(str, buff, fsize, &fsize, writelines, writeheader);
					printf("SUCCESS:Compiled File:%s\n", nicename.str_szBuffer);
				} catch(CompilerException err) {
					printf("ERROR:File:%s:Line:%d:Message:%s\n", nicename.str_szBuffer, err.CurrentLine, err.Message.str_szBuffer);
					delete[] buff;

					continue;
				}

				remove(outname.str_szBuffer);
				CFileStream ofs;
				ofs.Open(outname, "wb");
				ofs.Write(bytecode, fsize);
				ofs.Close();

				delete[] buff;
				delete[] bytecode;
			}

			if (shouldrun){
				printf("NOTICE:Running File:%s\n", nicename.str_szBuffer);

				Instance bsi;
				bsi.LoadDefaultLibaries();
				bsi.SetErrorCallback(errfunc);

				if (shoulddebug){
					if (bsi.Debug->Connect()){
						printf("NOTICE:Debugger connected\n");
					}else{
						printf("ERROR:Failed to attach debugger\n");
					}
				}

				try{
					if (shouldcompile) bsi.DoFile(outname.str_szBuffer);
					else bsi.DoFile(str.str_szBuffer);
				} catch(CompilerException err) {
					printf("ERROR:File:%s:Line:%d:Message:%s\n", err.CurrentFile.str_szBuffer, err.CurrentLine, err.Message.str_szBuffer);
					continue;
				}
			}
		}
	}

	while (sleepatend && fgetc(stdin) != '\n') {}

	return 0;
}